@@ -11,7 +11,6 @@ Inheritance diagram:
 # Distributed under the terms of the Modified BSD License.
 
 import abc
-import inspect
 import json
 import sys
 import traceback
@@ -22,79 +21,48 @@ from decorator import decorator
 from traitlets.config.configurable import Configurable
 from IPython.core.getipython import get_ipython
 from IPython.utils.sentinel import Sentinel
+from IPython.utils.dir2 import get_real_method
 from IPython.lib import pretty
 from traitlets import (
     Bool, Dict, Integer, Unicode, CUnicode, ObjectName, List,
     ForwardDeclaredInstance,
+    default, observe,
 )
 from IPython.utils.py3compat import (
     with_metaclass, string_types, unicode_type,
 )
 
 
-#-----------------------------------------------------------------------------
-# The main DisplayFormatter class
-#-----------------------------------------------------------------------------
-
-
-def _safe_get_formatter_method(obj, name):
-    """Safely get a formatter method
-    
-    - Classes cannot have formatter methods, only instance
-    - protect against proxy objects that claim to have everything
-    """
-    if inspect.isclass(obj):
-        # repr methods only make sense on instances, not classes
-        return None
-    method = pretty._safe_getattr(obj, name, None)
-    if callable(method):
-        # obj claims to have repr method...
-        if callable(pretty._safe_getattr(obj, '_ipython_canary_method_should_not_exist_', None)):
-            # ...but don't trust proxy objects that claim to have everything
-            return None
-        return method
-
-
 class DisplayFormatter(Configurable):
 
-    # When set to true only the default plain text formatter will be used.
-    plain_text_only = Bool(False, config=True)
-    def _plain_text_only_changed(self, name, old, new):
-        warnings.warn("""DisplayFormatter.plain_text_only is deprecated.
-        
-        It will be removed in IPython 5.0
-
-        Use DisplayFormatter.active_types = ['text/plain']
-        for the same effect.
-        """, DeprecationWarning)
-        if new:
-            self.active_types = ['text/plain']
-        else:
-            self.active_types = self.format_types
-    
-    active_types = List(Unicode(), config=True,
+    active_types = List(Unicode(),
         help="""List of currently active mime-types to display.
         You can use this to set a white-list for formats to display.
         
         Most users will not need to change this value.
-        """)
+        """).tag(config=True)
+
+    @default('active_types')
     def _active_types_default(self):
         return self.format_types
-    
-    def _active_types_changed(self, name, old, new):
+
+    @observe('active_types')
+    def _active_types_changed(self, change):
         for key, formatter in self.formatters.items():
-            if key in new:
+            if key in change['new']:
                 formatter.enabled = True
             else:
                 formatter.enabled = False
     
     ipython_display_formatter = ForwardDeclaredInstance('FormatterABC')
-    def _ipython_display_formatter_default(self):
+    @default('ipython_display_formatter')
+    def _default_formatter(self):
         return IPythonDisplayFormatter(parent=self)
     
     # A dict of formatter whose keys are format types (MIME types) and whose
     # values are subclasses of BaseFormatter.
     formatters = Dict()
+    @default('formatters')
     def _formatters_default(self):
         """Activate the default formatters."""
         formatter_classes = [
@@ -310,21 +278,21 @@ class BaseFormatter(Configurable):
     format_type = Unicode('text/plain')
     _return_type = string_types
 
-    enabled = Bool(True, config=True)
+    enabled = Bool(True).tag(config=True)
 
     print_method = ObjectName('__repr__')
 
     # The singleton printers.
     # Maps the IDs of the builtin singleton objects to the format functions.
-    singleton_printers = Dict(config=True)
+    singleton_printers = Dict().tag(config=True)
 
     # The type-specific printers.
     # Map type objects to the format functions.
-    type_printers = Dict(config=True)
+    type_printers = Dict().tag(config=True)
 
     # The deferred-import type-specific printers.
     # Map (modulename, classname) pairs to the format functions.
-    deferred_printers = Dict(config=True)
+    deferred_printers = Dict().tag(config=True)
     
     @catch_format_error
     def __call__(self, obj):
@@ -338,7 +306,7 @@ class BaseFormatter(Configurable):
             else:
                 return printer(obj)
             # Finally look for special method names
-            method = _safe_get_formatter_method(obj, self.print_method)
+            method = get_real_method(obj, self.print_method)
             if method is not None:
                 return method()
             return None
@@ -591,34 +559,34 @@ class PlainTextFormatter(BaseFormatter):
 
     # This subclass ignores this attribute as it always need to return
     # something.
-    enabled = Bool(True, config=False)
+    enabled = Bool(True).tag(config=False)
     
-    max_seq_length = Integer(pretty.MAX_SEQ_LENGTH, config=True,
+    max_seq_length = Integer(pretty.MAX_SEQ_LENGTH,
         help="""Truncate large collections (lists, dicts, tuples, sets) to this size.
         
         Set to 0 to disable truncation.
         """
-    )
+    ).tag(config=True)
     
     # Look for a _repr_pretty_ methods to use for pretty printing.
     print_method = ObjectName('_repr_pretty_')
 
     # Whether to pretty-print or not.
-    pprint = Bool(True, config=True)
+    pprint = Bool(True).tag(config=True)
 
     # Whether to be verbose or not.
-    verbose = Bool(False, config=True)
+    verbose = Bool(False).tag(config=True)
 
     # The maximum width.
-    max_width = Integer(79, config=True)
+    max_width = Integer(79).tag(config=True)
 
     # The newline character.
-    newline = Unicode('\n', config=True)
+    newline = Unicode('\n').tag(config=True)
 
     # format-string for pprinting floats
     float_format = Unicode('%r')
     # setter for float precision, either int or direct format-string
-    float_precision = CUnicode('', config=True)
+    float_precision = CUnicode('').tag(config=True)
 
     def _float_precision_changed(self, name, old, new):
         """float_precision changed, set float_format accordingly.
@@ -666,14 +634,17 @@ class PlainTextFormatter(BaseFormatter):
         self.float_format = fmt
 
     # Use the default pretty printers from IPython.lib.pretty.
+    @default('singleton_printers')
     def _singleton_printers_default(self):
         return pretty._singleton_pprinters.copy()
 
+    @default('type_printers')
     def _type_printers_default(self):
         d = pretty._type_pprinters.copy()
         d[float] = lambda obj,p,cycle: p.text(self.float_format%obj)
         return d
 
+    @default('deferred_printers')
     def _deferred_printers_default(self):
         return pretty._deferred_type_pprinters.copy()
 
@@ -904,7 +875,7 @@ class IPythonDisplayFormatter(BaseFormatter):
                 printer(obj)
                 return True
             # Finally look for special method names
-            method = _safe_get_formatter_method(obj, self.print_method)
+            method = get_real_method(obj, self.print_method)
             if method is not None:
                 method()
                 return True
